<#  
.SYNOPSIS  
		Returns current connection count for all frontend servers in a given pool
		including a breakdown of connection by client, frontend server and users.
 
		It can also be used to return connection information on an individual user.

.DESCRIPTION  
		This program will return a connection count for a given pool. The program
		can be edited to set a default pool. You will also be able to get
		information on an individual user by providing the users SamAccountName.
		As well as listing all the connected users to the default pool.
		
		NOTE: In order to gain remote access to each frontend server's 
		RTCLOCAL database where connection information is found, 
		you need to open the local firewall for port 1434.
		Also, need to go into the SQL Server Configuration Manager 
		and for RTCLOCAL, enable named pipes and restart the SQL 
		service for the named pipes to take effect.

		Port 1434 is required in order to make the connection to 
		the named instance RTCLOCAL on the remote machines.

.NOTES  
    Version      	   	: 1.9 - See changelog at http://www.ehloworld.com/681
		Wish list					: Could add # users who are enterprise voice enabled and other individual stats to this page as well
											: check for firewall rules
    Rights Required		: TBD
    Sched Task Req'd	: No
    Lync Version			: 2010 through CU6
    Author(s)    			: Tracy A. Cerise (tracy@uky.edu)
    									: Mahmoud Badran (v-mabadr@microsoft.com)
    									: Pat Richard (pat@innervation.com)
    Dedicated Post		: http://www.ehloworld.com/269
    Disclaimer   			: You running this script means you won't blame me if this breaks your stuff.
    Info Stolen from 	: http://blogs.technet.com/b/meacoex/archive/2011/07/19/list-connections-and-users-connected-to-lync-registrar-pool.aspx
    Acknowledgement		: This program's database connection information was originally taken from the "List Connections to Registrar Pools" submitted by Scott Stubberfield and Nick Smith from Microsoft to the Lync 2010 PowerShell blog  (http://blogs.technet.com/b/csps/) on June 10, 2010.

.LINK  
    http://www.ehloworld.com/269

.EXAMPLE
		.\Get-CsConnections.ps1 
 
		Description
		-----------
		Returns information on all connections on all frontend servers in the 
		default pool that has been hard-coded into the program. Or, if no hard coded value exists, 
		and only one registrar exists, that pool is used.

.EXAMPLE
		.\Get-CsConnections.ps1 -Pool [pool FQDN]
 
		Description
		-----------
		Returns information on all connections on all frontend servers in the 
		pool FQDN given with the pool parameter.

.EXAMPLE
		.\Get-CsConnections.ps1 -Pool [pool name]
 
		Description
		-----------
		Returns information on all connections on all frontend servers in the 
		pool NetBIOS name given with the pool parameter. The script will append
		the %userdnsdomain% to build the FQDN.
		
.EXAMPLE
		.\Get-CsConnections.ps1 -SIPAddr userid@sip.domain
 
		Description
		-----------
		Returns all connection information for the given user including which 
		frontend server connected to, how many connections and which clients
		connected with.
 
.EXAMPLE
		.\Get-CsConnections.ps1 -FilePath c:\path\to\file\filename.csv
 
		Description
		-----------
		Returns information on all connections on all frontend servers in the 
		default pool that has been hard-coded into the program and in addition
		writes out all the raw connection information into the filename specified.

.EXAMPLE
		.\Get-CsConnections.ps1 -IncludeUsers
 
		Description
		-----------
		Returns information on all connections on all frontend servers in the 
		default pool that has been hard-coded into the program and in addition
		writes out all the users connection information. This does NOT include
		system related connections.

.EXAMPLE
		.\Get-CsConnections.ps1 -IncludeUsers -UserHighConnectionFlag 5
 
		Description
		-----------
		Returns information on all connections on all frontend servers in the 
		default pool that has been hard-coded into the program and in addition
		writes out all the users connection information. This does NOT include
		system related connections. The UserHighConnectionFlag integer will color
		all users with that many (or more) connections in red. The default is 3.

.EXAMPLE
		.\Get-CsConnections.ps1 -IncludeUsers -IncludeSystem
 
		Description
		-----------
		Returns information on all connections on all frontend servers in the 
		default pool that has been hard-coded into the program and in addition
		writes out all the users connection information. This includes any system
		related connections.

.EXAMPLE
		.\Get-CsConnections.ps1 -Pool [pool FQDN] -ClientVersion [version number]
 
		Description
		-----------
		Returns all connection information for only clients that contain the specified client version.
		Also lists the users connecting with that version (same as -IncludeUsers option).		

.EXAMPLE
		.\Get-CsConnections.ps1 -Pool [pool FQDN] -ShowFullClient
 
		Description
		-----------
		Returns extended information for clients including mobile device OS and type.
#>
#Requires -Version 2.0

[CmdletBinding(SupportsShouldProcess = $True)]
param(
	[parameter(ValueFromPipeline = $false, ValueFromPipelineByPropertyName = $true, Mandatory = $false)] 	
	[string]$Pool, 
	[parameter(ValueFromPipeline = $false, ValueFromPipelineByPropertyName = $true, Mandatory = $false)] 
	[string]$SIPAddr, 
	[parameter(ValueFromPipeline = $false, ValueFromPipelineByPropertyName = $true, Mandatory = $false)] 
	[string]$FilePath, 
	[parameter(ValueFromPipeline = $false, ValueFromPipelineByPropertyName = $true, Mandatory = $false)] 
	[switch]$IncludeUsers,
	[parameter(ValueFromPipeline = $false, ValueFromPipelineByPropertyName = $true, Mandatory = $false)] 
	[switch]$IncludeSystem,
	[parameter(ValueFromPipeline = $false, ValueFromPipelineByPropertyName = $true, Mandatory = $false)] 
	[ValidateRange(2,10)]
	[int]$UserHighConnectionFlag = 4,
	[parameter(ValueFromPipeline = $false, ValueFromPipelineByPropertyName = $true, Mandatory = $false)] 
	[string]$ClientVersion,
	[parameter(ValueFromPipeline = $false, ValueFromPipelineByPropertyName = $true, Mandatory = $false)] 
	[switch]$ShowFullClient,
	[parameter(ValueFromPipeline = $false, ValueFromPipelineByPropertyName = $true, Mandatory = $false)] 
	[string]$Server
)

#################################################################################################
##########################################  Functions  ##########################################
#################################################################################################
function Get-Data {
	[CmdletBinding(SupportsShouldProcess = $True)] 
	param (
	 [string]$sipAddr, 
	 [string]$Server,
	 [string]$ClientVersion
	)
 
	##############################################################################################
	# Went to using a named parameter for this function due to the 
	# way Powershell does its thing with parameter passing, which 
	# is NOT GOOD!  At any rate, need to call this function
	# as you would from a command line: Get-Data -sipAddr "value" 
	# -server "value"
	#
	# Also, assuming a value of NULL for the SIP address of an 
	# individual user, mostly to use this for finding overall 
	# values, only occasionally to seek specific users.
	##############################################################################################
 
	if ($sipAddr) {
		[string]$whereClause = "where R.UserAtHost = '$sipAddr' "
	} else {
		if ($IncludeSystem){
			[string]$whereClause = $null
		}elseif ($ClientVersion){			
			[string]$whereClause = "where RE.ClientApp like '%$ClientVersion%' and R.UserAtHost not like 'RtcApplication-%' "			
		}else{
			[string]$whereClause = "where R.UserAtHost not like 'RtcApplication-%' "
		}
	}
 
	#Define SQL Connection String
	[string]$connstring = "server=$server\rtclocal;database=rtcdyn;trusted_connection=true;"
 
	#Define SQL Command
	[object]$command = New-Object System.Data.SqlClient.SqlCommand
 
	<#
	$command.CommandText = "Select (cast (RE.ClientApp as varchar (100))) as ClientVersion, R.UserAtHost as UserName, Reg.Fqdn `
									From rtcdyn.dbo.RegistrarEndpoint RE `
									Inner Join `
									rtc.dbo.Resource R on R.ResourceId = RE.OwnerId `
									Inner Join `
									rtcdyn.dbo.Registrar Reg on Reg.RegistrarId = RE.PrimaryRegistrarClusterId `
									$whereClause `
									Order By ClientVersion, UserName"
	#>								
	$command.CommandText = "Select (cast (RE.ClientApp as varchar (100))) as ClientVersion, R.UserAtHost as UserName, FE.Fqdn `
	From rtcdyn.dbo.RegistrarEndpoint RE `
	Inner Join rtcdyn.dbo.Endpoint EP on RE.EndpointId = EP.EndpointId `
	Inner Join rtc.dbo.Resource R on R.ResourceId = RE.OwnerId `
	Inner Join rtcdyn.dbo.FrontEnd FE on EP.RegistrarId = FE.FrontEndId `
	$whereClause `
	Order By ClientVersion, UserName"
 
	[object]$connection = New-Object System.Data.SqlClient.SqlConnection
	$connection.ConnectionString = $connstring
	$connection.Open()
 
	$command.Connection = $connection
 
	[object]$sqladapter = New-Object System.Data.SqlClient.SqlDataAdapter
	$sqladapter.SelectCommand = $command
 
	[object]$results = New-Object System.Data.Dataset
	$recordcount = $sqladapter.Fill($results)
	$connection.Close()
	return $Results.Tables[0]
} # end function Get-Data

function Set-ModuleStatus { 
	[CmdletBinding(SupportsShouldProcess = $True)]
	param	(
		[parameter(ValueFromPipeline = $true, ValueFromPipelineByPropertyName = $true, Mandatory = $true, HelpMessage="No module name specified!")] 
		[ValidateNotNullOrEmpty()]
		[string]$name
	)
	if(!(Get-Module -name "$name")) { 
		if(Get-Module -ListAvailable | ? {$_.name -eq "$name"}) { 
			Import-Module -Name "$name" 
			Write-Verbose "$name module was imported"
			return $true | Out-Null
		} else {
			Write-Verbose "$name module was not available"
			return $false | Out-Null
		}
	}else {
		Write-Verbose "$name module already imported"
		return $true | Out-Null
	}
} # end function Set-ModuleStatus

function Remove-ScriptVariables {  
	[CmdletBinding(SupportsShouldProcess = $True)]
	param(
		[parameter(ValueFromPipeline = $true, ValueFromPipelineByPropertyName = $true, Mandatory = $true, HelpMessage="No module name specified!")] 
		[ValidateNotNullOrEmpty()]
		[string]$Path
	)	
  
	$result = Get-Content $path |  
	ForEach { 
		if ( $_ -match '(\$.*?)\s*=') {      
			$matches[1]  | ? { $_ -notlike '*.*' -and $_ -notmatch 'result' -and $_ -notmatch 'env:'}  
		}  
	}  
	ForEach ($v in ($result | Sort-Object | Get-Unique)){		
		# Write-Verbose "Removing" $v.replace("$","")
		Remove-Variable ($v.replace("$","")) -ea 0
	}
} # end function Remove-ScriptVariables

#################################################################################################

Set-ModuleStatus -name Lync

if ((! $server) -and (! $pool) -and (((Get-CsService -Registrar) | measure).count -eq 1)){
	Write-Verbose "Retrieving pool info"
	$pool = (Get-CsService -registrar).poolfqdn
	Write-Verbose "Pool is now set to $pool"
}
if ((! $server) -and (! $pool)){
	$Pool = Read-Host "Enter front end pool FQDN"
	Write-Verbose "Pool is now set to $pool"
}

#################################################################################################
########################################  Main Program  #########################################
#################################################################################################
# Here is where we pull all the frontend server(s) from our topology for the designated
# pool and iterate through them to get current connections from all the servers.
#
# There are several possibilities here:
#	 1. Have collection of frontend servers
#	 2. Have a single frontend server or
#	 3. Have no servers
#  4. User specified a server instead of a pool

if ($pool -and (! $server)){
	# set FQDN if $pool was set to just the netbios name
	if (!($pool -match "\.")){
		Write-Verbose "No FQDN supplied. Building it."
		# $pool = $pool+"."+$env:userdnsdomain
		$pool = $pool+"."+([System.DirectoryServices.ActiveDirectory.Domain]::GetCurrentDomain()).name
	}
	$feServers = Get-CsComputer -Pool $Pool | Sort identity
	Write-Host "`n`nChecking these pool servers in pool: " -ForegroundColor cyan -NoNewLine
	Write-Host $pool
	ForEach ($server in $feservers){
		Write-Host $Server.identity -ForegroundColor yellow
	}
}elseif ($server -and (! $pool)){
	if (!($server -match "\.")){
		Write-Verbose "No FQDN supplied. Building it."
		$server = $server+"."+$env:userdnsdomain
	}
	$feServers = Get-CsComputer -identity $server -ea 0
	Write-Host "`n`nChecking this server: " -ForegroundColor cyan -NoNewLine
	Write-Host $server
}

if ($feServers.count) {
	# Frontend pool collection, iterate through them
	for ($i = 0; $i -lt $feServers.count; $i++) {
		if ($SIPAddr) {			
			if ((!($SIPAddr -match "@"))-and ((Get-CsSipDomain | measure).count -eq 1)){
				Write-Verbose "Incomplete SIP address supplied. Building it."
				$SipAddr = $SipAddr+"@"+(Get-CsSipDomain).identity
				Write-Verbose "SIP address corrected to $SipAddr"
			}
			Write-Verbose "Gathering info from $feServers[$i].identity"
			$data = Get-Data -sipAddr $SIPAddr -server $feServers[$i].identity -ClientVersion $ClientVersion
		} else {
			Write-Verbose "Gathering info from $feServers[$i].identity"
			$data = Get-Data -server $feServers[$i].identity -ClientVersion $ClientVersion
		}
 
		# Since an individual user's connections are all homed on one server, 
		# we won't have data coming back from all frontend servers in the 
		# case of searching for a single user
		if ($data) {
			$overallrecords = $overallrecords + $data
		}
	}
} elseif ($feServers) {
	# Have a standalone server or a FE pool of only one server
	if ($SIPAddr) {
		if ((!($SIPAddr -match "@"))-and ((Get-CsSipDomain | measure).count -eq 1)){
			Write-Verbose "Incomplete SIP address supplied. Building it."
			$SipAddr = $SipAddr+"@"+(Get-CsSipDomain).identity
			Write-Verbose "SIP address corrected to $SipAddr"
		}
		$data = Get-Data -sipAddr $SIPAddr -server $feServers.identity -ClientVersion $ClientVersion
	} else {
		Write-Verbose "Gathering info from $feServers.identity"
		$data = Get-Data -server $feServers.identity -ClientVersion $ClientVersion
	}
 
	# Make sure we have data to work with...
	if ($data) {
		$overallrecords = $data
	}
}else{
	Write-Host "No servers returned!" -ForegroundColor Red
}
 
# Check to see if we have any data to act on
if (! $overallrecords) {
	Write-Host -ForegroundColor Yellow "`r`nNothing returned from query!`r`n"
 
	# Nothing else to do
	exit
} else {
	$count = 0 
	$userHash = @{}
	$clientHash = @{}
	$serverHash = @{}
	$userlist = @{}
 
	$overallrecords | ForEach-object {
		# Each record has three components: Connected Client Version, User's SIP
		# address and the frontend server's FQDN. Here, we'll build a hash
		# for each of these components for each record.
 
		# Build hash of users

 		$userlist = $_.UserName

		if (! $userHash.ContainsKey($_.UserName)) {
			$userHash.add($_.UserName, 1)
		} else {
			$userHash.set_item($_.UserName, ($userHash.get_item($_.UserName) + 1))
		}
 
		# Build hash of servers
		if (! $serverHash.ContainsKey($_.fqdn)) {
			$serverHash.add($_.fqdn, 1)
		} else {
			$serverHash.set_item($_.fqdn, ($serverHash.get_item($_.fqdn) + 1))
		}
 
		# Build hash of clients
		# Lets get rid of the extraneous verbage from the client version names, if applicable
		if ($_.ClientVersion.contains('(') -and (! $ShowFullClient)) {
			# Get rid of extraneous verbage
			$clientName = $_.ClientVersion.substring(0, $_.ClientVersion.IndexOf('('))			
		} else {
			# Have a client name with no extraneous verbage or $ShowFullClient switch specified
			$clientName = $_.ClientVersion
			$clientName = $clientName.replace("Microsoft ","")
			$clientName = $clientName.replace("Office ","")
			$clientName = $clientName.replace("AndroidLync","Android")
			$clientName = $clientName.replace("iPadLync","iPad")
			$clientName = $clientName.replace("iPhoneLync","iPhone")
			$clientName = $clientName.replace("WPLync","WP")
		}
 
		if (! $clientHash.ContainsKey($clientName)) {
			$clientHash.add($clientName, 1)
		} else {
			$clientHash.set_item($ClientName, ($clientHash.get_item($ClientName) + 1))
		}
		$count++
	}
}
 
#################################################################################################
####################################  Output Query Results  #####################################
#################################################################################################
# If output to file is chosen, then write out the results and a note to that effect
# then exit
 
if ($FilePath) {
	$overallrecords | Export-Csv $FilePath 
	Write-Host -ForegroundColor green "`r`nQuery Results written to $FilePath`r`n" 
	exit
}

# Client Version, Agent, Connections
Write-Host "`r`n`r`n"
if (! $ShowFullClient){
	Write-Host ("{0, -22}{1, -42}{2, 14}" -f "Client Version", "Agent", "Connections") -ForegroundColor cyan 
}else{
	Write-Host ("{0, -64}{1, 14}" -f "Agent", "Connections") -ForegroundColor cyan 
}
Write-Host "------------------------------------------------------------------------------" -ForegroundColor cyan
ForEach ($key in $clientHash.keys | sort -descending) {
	# Break down client version into its two component parts and print
	# them out along with their respective counts in a nice format
	$index = $key.indexof(" ")
 
	if ($index -eq "-1") {
		# No second part
		$first = $key
		$second = " "
	} else {
		# Client version/agent has two main parts
		$first = $key.substring(0, $index)
		$second = $key.substring($index + 1)
	}
 
	$value = $clientHash.$key
	if (! $ShowFullClient){
		"{0,-22}{1,-45}{2,9}" -f $first, $second, $value
	}else{
		"{0,-73}{1,4}" -f $second, $value
	}
}
Write-Host "------------------------------------------------------------------------------" -ForegroundColor cyan 

# Frontend Server, Connections
Write-Host "`r`n`r`n"
Write-Host ("{0,-41}{1,15}" -f "Frontend Server", "Connections") -ForegroundColor cyan 
Write-Host "--------------------------------------------------------" -ForegroundColor cyan 
ForEach ($key in ($serverHash.keys | sort)) {
	$value = $serverHash.$key
	[string]$Percent = "("+"{0:P2}" -f ($value/$count)+")"
	"{0,-40}{1,6} {2,9}" -f $key, $value, $Percent
}
Write-Host "--------------------------------------------------------" -ForegroundColor cyan 
"{0,-40}{1,6}" -f "Total connections", $count 

# Unique Users, Unique Clients
Write-Host "`r`n`r`n"
Write-Host ("{0,-41}{1,15}" -f "Total Unique Users/Clients", "Total") -ForegroundColor cyan 
Write-Host "--------------------------------------------------------" -ForegroundColor cyan 
"{0,-41}{1,15}" -f "Unique Users", $userHash.count
"{0,-41}{1,15}" -f "Client Versions", $clientHash.count
Write-Host "--------------------------------------------------------" -ForegroundColor cyan 

# Users, Connections
if ($IncludeUsers){
	Write-Host "`r`n`r`n"
	Write-Host ("{0,-32}{1,-5}" -f "Connected Users", "Connections") -ForegroundColor cyan
	Write-Host "--------------------------------------------------------" -ForegroundColor cyan
	ForEach ($key in $userHash.keys | sort) {
		$value = $userHash.$key
		if ($value -ge $UserHighConnectionFlag){
 			Write-Host ("{0,-37}{1,-5}" -f $key, $value) -ForegroundColor red
		}else{
			"{0,-37}{1,-5}" -f $key, $value
		}
	}
	Write-Host "--------------------------------------------------------`r`n" -ForegroundColor cyan 
}

Write-Verbose "Query complete"

Remove-ScriptVariables($MyInvocation.MyCommand.Name)
# SIG # Begin signature block
# MIIQtAYJKoZIhvcNAQcCoIIQpTCCEKECAQExCzAJBgUrDgMCGgUAMGkGCisGAQQB
# gjcCAQSgWzBZMDQGCisGAQQBgjcCAR4wJgIDAQAABBAfzDtgWUsITrck0sYpfvNR
# AgEAAgEAAgEAAgEAAgEAMCEwCQYFKw4DAhoFAAQUUx3tUQwZyZqMhyoxlGKcRjgn
# j6mggg4EMIIGtTCCBZ2gAwIBAgIQDdDjN0rJW9v6a0NLKkjsBjANBgkqhkiG9w0B
# AQUFADBsMQswCQYDVQQGEwJVUzEVMBMGA1UEChMMRGlnaUNlcnQgSW5jMRkwFwYD
# VQQLExB3d3cuZGlnaWNlcnQuY29tMSswKQYDVQQDEyJEaWdpQ2VydCBIaWdoIEFz
# c3VyYW5jZSBFViBSb290IENBMB4XDTEyMDQxODEyMDAwMFoXDTI3MDQxODEyMDAw
# MFowZTELMAkGA1UEBhMCVVMxFTATBgNVBAoTDERpZ2lDZXJ0IEluYzEZMBcGA1UE
# CxMQd3d3LmRpZ2ljZXJ0LmNvbTEkMCIGA1UEAxMbRGlnaUNlcnQgRVYgQ29kZSBT
# aWduaW5nIENBMIIBIjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKCAQEAuQZ0HF20
# IKqpIagqQkarJSAXJcsij5CioDFrgwV1r7IOfBJJe2qGZIQPg9xkubFuFgU+HJW5
# 5+eIbbhigZB51N314pb5w7WII1dKGs9xKekIAI+1mOOnMv2sLrj0k1P0CjlDka/V
# a+jUn0a9jj2r4vkr1OoAQGYkt+h/tER1jXiarjHBN89OH1v4RUrXP8LJkgZkvt4G
# iq/Q6Iqx8CyIAG8L3IWnTMsGv9YuKjJuKXGvjiLzD9DYmEgtqAjLtosjwmPgtnPr
# b30mT4v3ND03hgy3eCf0wobbQ2ta+D099OiwYlbG5+14ofv9enJPMmXEfMPEd6AE
# MjLtjz+vht1+0QIDAQABo4IDWDCCA1QwEgYDVR0TAQH/BAgwBgEB/wIBADAOBgNV
# HQ8BAf8EBAMCAYYwEwYDVR0lBAwwCgYIKwYBBQUHAwMwfwYIKwYBBQUHAQEEczBx
# MCQGCCsGAQUFBzABhhhodHRwOi8vb2NzcC5kaWdpY2VydC5jb20wSQYIKwYBBQUH
# MAKGPWh0dHA6Ly9jYWNlcnRzLmRpZ2ljZXJ0LmNvbS9EaWdpQ2VydEhpZ2hBc3N1
# cmFuY2VFVlJvb3RDQS5jcnQwgY8GA1UdHwSBhzCBhDBAoD6gPIY6aHR0cDovL2Ny
# bDMuZGlnaWNlcnQuY29tL0RpZ2lDZXJ0SGlnaEFzc3VyYW5jZUVWUm9vdENBLmNy
# bDBAoD6gPIY6aHR0cDovL2NybDQuZGlnaWNlcnQuY29tL0RpZ2lDZXJ0SGlnaEFz
# c3VyYW5jZUVWUm9vdENBLmNybDCCAcQGA1UdIASCAbswggG3MIIBswYJYIZIAYb9
# bAMCMIIBpDA6BggrBgEFBQcCARYuaHR0cDovL3d3dy5kaWdpY2VydC5jb20vc3Ns
# LWNwcy1yZXBvc2l0b3J5Lmh0bTCCAWQGCCsGAQUFBwICMIIBVh6CAVIAQQBuAHkA
# IAB1AHMAZQAgAG8AZgAgAHQAaABpAHMAIABDAGUAcgB0AGkAZgBpAGMAYQB0AGUA
# IABjAG8AbgBzAHQAaQB0AHUAdABlAHMAIABhAGMAYwBlAHAAdABhAG4AYwBlACAA
# bwBmACAAdABoAGUAIABEAGkAZwBpAEMAZQByAHQAIABDAFAALwBDAFAAUwAgAGEA
# bgBkACAAdABoAGUAIABSAGUAbAB5AGkAbgBnACAAUABhAHIAdAB5ACAAQQBnAHIA
# ZQBlAG0AZQBuAHQAIAB3AGgAaQBjAGgAIABsAGkAbQBpAHQAIABsAGkAYQBiAGkA
# bABpAHQAeQAgAGEAbgBkACAAYQByAGUAIABpAG4AYwBvAHIAcABvAHIAYQB0AGUA
# ZAAgAGgAZQByAGUAaQBuACAAYgB5ACAAcgBlAGYAZQByAGUAbgBjAGUALjAdBgNV
# HQ4EFgQUrWkGcPyAGxazqRiUa5QChl73J4wwHwYDVR0jBBgwFoAUsT7DaQP4v0cB
# 1JgmGggC72NkK8MwDQYJKoZIhvcNAQEFBQADggEBAJ5bljouEoisqwFtpJ915AGH
# o6Uy17y6qX6j1hQX98ITa3xzjytq5Q8mWWiwjiWbbO/6bJOSCMFNz0WenEbWHnSh
# mxSj+gEvSrEB4XJASBETaLk2nZFL18I5EhDBxNy7YhQUKmFdTzh8Zh/GG/+tvk9/
# lFtzQwAPTXO3Uc8O9nfAW800jNljE6oOYRHW8o4n/LR7uLkRIJGGeOoO1Cj/KtUk
# OOg3suyWu5+8ShZQ4V6/UX0joDLHwZSeesnAJqLMJYegEn50ny2NscjnhL650ene
# u2pOiHNx4SI4yySH6XN+UbL/mOtOfi/gyg76s17RugVCqEifg/Y/xMqo32igUGEw
# ggdHMIIGL6ADAgECAhADqMSsegNouSUcbjUZwgbhMA0GCSqGSIb3DQEBBQUAMGUx
# CzAJBgNVBAYTAlVTMRUwEwYDVQQKEwxEaWdpQ2VydCBJbmMxGTAXBgNVBAsTEHd3
# dy5kaWdpY2VydC5jb20xJDAiBgNVBAMTG0RpZ2lDZXJ0IEVWIENvZGUgU2lnbmlu
# ZyBDQTAeFw0xMjA4MzEwMDAwMDBaFw0xMzA5MDUxMjAwMDBaMIH3MR0wGwYDVQQP
# DBRQcml2YXRlIE9yZ2FuaXphdGlvbjETMBEGCysGAQQBgjc8AgEDEwJVUzEZMBcG
# CysGAQQBgjc8AgECEwhNaWNoaWdhbjEPMA0GA1UEBRMGRDMwMDc5MRowGAYDVQQJ
# ExExMjc3MyBXaW5kc29yIEN0LjEOMAwGA1UEERMFNDgzMTMxCzAJBgNVBAYTAlVT
# MQswCQYDVQQIEwJNSTEZMBcGA1UEBxMQU3RlcmxpbmcgSGVpZ2h0czEZMBcGA1UE
# ChMQSW5uZXJ2YXRpb24sIExMQzEZMBcGA1UEAxMQSW5uZXJ2YXRpb24sIExMQzCC
# ASIwDQYJKoZIhvcNAQEBBQADggEPADCCAQoCggEBAMxeMd353M1Z72cYQMCUcQYQ
# 98yHjnBGjtdvsUTXrurr6jRSMeM1jvxscCnbcVGqzDyCE5f5LMmAR5TCv92iJ0Gk
# ltGbg6JQoMnZ5CimtgID8VkpGjmN841Mf1I033cjj6edp+h1j2QZz9MBTgWJ+apJ
# DEHpNO3TXd9FotW/oGpJW0BWCHy+LZ2MBMOkuf2p6CPPTnN5qqtS6etOHMr8gYJI
# RMzMHSAIRXqPmqjvjQOoJexA4lNPwyK4Dm2Cd559WB+pzLeEe5APP5qfPFXZZDkI
# TNneb1DDVTOblwjOfUFZwh0n8/A4e8eeSYaBqPCO8Ahs75x87zeV8VnoqQKA4mEC
# AwEAAaOCA14wggNaMB8GA1UdIwQYMBaAFK1pBnD8gBsWs6kYlGuUAoZe9yeMMB0G
# A1UdDgQWBBRsdVoCcikQTAsLEOpuVPd/GHXZvjAtBgNVHREEJjAkoCIGCCsGAQUF
# BwgDoBYwFAwSVVMtTUlDSElHQU4tRDMwMDc5MA4GA1UdDwEB/wQEAwIHgDATBgNV
# HSUEDDAKBggrBgEFBQcDAzBzBgNVHR8EbDBqMDOgMaAvhi1odHRwOi8vY3JsMy5k
# aWdpY2VydC5jb20vRVZDb2RlU2lnbmluZy1nMS5jcmwwM6AxoC+GLWh0dHA6Ly9j
# cmw0LmRpZ2ljZXJ0LmNvbS9FVkNvZGVTaWduaW5nLWcxLmNybDCCAcQGA1UdIASC
# AbswggG3MIIBswYJYIZIAYb9bAMCMIIBpDA6BggrBgEFBQcCARYuaHR0cDovL3d3
# dy5kaWdpY2VydC5jb20vc3NsLWNwcy1yZXBvc2l0b3J5Lmh0bTCCAWQGCCsGAQUF
# BwICMIIBVh6CAVIAQQBuAHkAIAB1AHMAZQAgAG8AZgAgAHQAaABpAHMAIABDAGUA
# cgB0AGkAZgBpAGMAYQB0AGUAIABjAG8AbgBzAHQAaQB0AHUAdABlAHMAIABhAGMA
# YwBlAHAAdABhAG4AYwBlACAAbwBmACAAdABoAGUAIABEAGkAZwBpAEMAZQByAHQA
# IABDAFAALwBDAFAAUwAgAGEAbgBkACAAdABoAGUAIABSAGUAbAB5AGkAbgBnACAA
# UABhAHIAdAB5ACAAQQBnAHIAZQBlAG0AZQBuAHQAIAB3AGgAaQBjAGgAIABsAGkA
# bQBpAHQAIABsAGkAYQBiAGkAbABpAHQAeQAgAGEAbgBkACAAYQByAGUAIABpAG4A
# YwBvAHIAcABvAHIAYQB0AGUAZAAgAGgAZQByAGUAaQBuACAAYgB5ACAAcgBlAGYA
# ZQByAGUAbgBjAGUALjB5BggrBgEFBQcBAQRtMGswJAYIKwYBBQUHMAGGGGh0dHA6
# Ly9vY3NwLmRpZ2ljZXJ0LmNvbTBDBggrBgEFBQcwAoY3aHR0cDovL2NhY2VydHMu
# ZGlnaWNlcnQuY29tL0RpZ2lDZXJ0RVZDb2RlU2lnbmluZ0NBLmNydDAMBgNVHRMB
# Af8EAjAAMA0GCSqGSIb3DQEBBQUAA4IBAQBPxrm+zwZtm2e/V6fkoZUJ8ABogI90
# wczn7qzS2FZVwn0pJsvKNLQngSv2WpZPa0BKK1J9txsv9eJOkpagKGBUo0mDo6Kh
# GvIng22gNGksbfvI7C+agDmjst3uHs2Vsr/m7e7hidrmFJRXpxE8hYJK/MpUDadW
# mZm2VKKrzv+ITuR3lpbjgTzAwBpP4OPedvYg5TQl9OxdQgFD1OtG0WLNUlzx7J1h
# 67B2zDilwSKVzvwYUCN/lEHOC6q3XPpOCp300of8B8Hq84YQ3HbcsZpIgtSaGxwp
# okt2HBlBgkynwPkx29URACt3gRX/3CokssVuiZLKjVwR6/PVYujf+j80MYICGjCC
# AhYCAQEweTBlMQswCQYDVQQGEwJVUzEVMBMGA1UEChMMRGlnaUNlcnQgSW5jMRkw
# FwYDVQQLExB3d3cuZGlnaWNlcnQuY29tMSQwIgYDVQQDExtEaWdpQ2VydCBFViBD
# b2RlIFNpZ25pbmcgQ0ECEAOoxKx6A2i5JRxuNRnCBuEwCQYFKw4DAhoFAKB4MBgG
# CisGAQQBgjcCAQwxCjAIoAKAAKECgAAwGQYJKoZIhvcNAQkDMQwGCisGAQQBgjcC
# AQQwHAYKKwYBBAGCNwIBCzEOMAwGCisGAQQBgjcCARUwIwYJKoZIhvcNAQkEMRYE
# FNeWftfO3d39B0LIbFxRaFtwbtxuMA0GCSqGSIb3DQEBAQUABIIBAG4JYWQD7Xv/
# 4OwIhY1dOZRQxdEIw2djDel+DwQKMjjdoGaTEyPUU+1ULON6iXE4pEelX4BUoHK/
# 3ZOGCV3+2bCl93EfszN9qWi/aBRH6LunSvPDzPvdqhiebsKttAjFfkmwIeWAXGl2
# IVCW6lChbY7thJWaGQIz4Vvtuk6MZtiNOyyoYsJKPaOi0YWZkv/E/88pyuskv/6R
# +blZay8eQAES9X3PGhT3ts0EEzfXdvbS+1LlsK4jqeh8lUm2ItxbxicEpU2nDgzg
# wUAPgmHy/3dZaHSlZr6M9SB1kRmI80dzaf3Rr2WIuGhsvcJdW2TQ0Jqn5sNoRc+S
# rM+W4/ikhFo=
# SIG # End signature block
